// Algorithm.h : Declaration of the Algorithm class
//
// 11/19/03 Rich Williams - modified for use in SEEK
//

#ifndef __ALGORITHM_H_
#define __ALGORITHM_H_

#include <scew.h>

#include "RuleSet.h"
#include "Utilities.h"

class EnvCellSet;
class EnvLayerSet;
class EnvLayer;

// ========================================================================
// Algorithm
// ========================================================================
class Algorithm
{
public:
  Algorithm();
  virtual ~Algorithm();

  virtual char * type() { return "Algorithm"; }

  virtual void iterate() = 0;
  virtual bool done() = 0;

  virtual void saveText(char * strFilename) = 0;
  virtual void loadText(char * strFilename) = 0;

  virtual char * toXML() = 0;
  virtual char * toXMLWithTokens() { return NULL; };
  virtual char * toXMLOnlyResults() = 0;
  virtual void fromXML(char * strXML) = 0;

  virtual void fromXMLFile(char * strXMLFile) = 0;
  virtual void toXMLFile(char * strXMLFile) = 0;

  virtual char * getParameter(char * sParamName) = 0;
  virtual void setParameter(char * sParamName, char * sParamValue) = 0;


  // TODO: create get/set methods to remove direct access using friend classes
#ifdef __GARPWIN
  friend class CAlgorithm;
#endif
};

// ========================================================================
// GarpAlgorithm
// ========================================================================
class GarpAlgorithm : public Algorithm
{
private:
  friend class CAlgorithm;
  friend class DataContainer;

  // current ruleset
  RuleSet objNew;

  // best model so far
  RuleSet objBest;

  // layer used to generate the model
  EnvLayerSet * objModelLayerSet;

  // data points used for training
  EnvCellSet * objTrainSet;

  // indicates when the above datasets can be deallocated in destructor
  bool bAutoDeleteDatasets;


public:
  GarpAlgorithm();
  virtual ~GarpAlgorithm();

  virtual char * type() { return "GarpAlgorithm"; }

  void initialize(EnvLayerSet * objLyrSet, EnvCellSet * objTrnSet);
  void iterate();
  bool done();

  void loadText(char * strFilename);
  void saveText(char * strFilename);

  void fromXML(char * strXML);
  char * toXML();
  char * toXMLWithTokens();
  char * toXMLOnlyResults();

  void fromXMLFile(char * strXMLFile);
  void toXMLFile(char * strXMLFile);
  void fromXML(scew_element const * pXMLDocNode);

  RuleSet * getBestSet() { return &objBest; }
  void getInitialModel(int intSize, EnvCellSet * objTrainSet);
  void updateRuleUsage();

  int ruleSetSize()
  { return objBest.size(); }

  char * getParameter(char * sParamName);
  void setParameter(char * sParamName, char * sParamValue);

  char * getSelectedLayersAsString();
  void setSelectedLayers(char * strParamValue);

private:
  // private methods
  void initializeProperties();

  // Garp algorithm
  void DisplayStatus();
  void colonize(RuleSet * objRules, EnvCellSet * objTrainSet, int intNewRules);
  void evaluate(RuleSet * objRules, EnvCellSet * objTrainSet);
  int saveRule(int iIndex);
  void measure();
  double converge();
  void select();

  void mutate();
  void crossover();
  void join();

  void updateHeuOpPerformance(char chrType);

  void generate(EnvCellSet * objTestDataset);

  scew_element const * getRuleSetXMLById(scew_element const * pXMLNode, char * sId);
  char * parametersToXML(bool bWithTokens);
  char * toXMLWithParameters(char * sParams, bool bResultXMLOnly);

  //void resetActiveGenes(EnvLayerSet * objLayerSet);
  void resetActiveGenes(EnvCellSet * objCellSet);

  void parametersFromXML(scew_element const * pXMLMainNode);

  // deprecated methods
  void saveBestRules(RuleSet * toRuleSet, RuleSet * fromRuleSet);
  void concatenateRuleSets(RuleSet * toRuleSet, RuleSet * fromRuleSet);


private:

  long lVersion;

  // flags
  int Sigflag;
  int Postflag;
  int Compflag;
  int	Adjustflag;

  int BioclimOnlyFlag;
  int LogitOnlyFlag;
  int RangeRuleFlag;
  int NegatedRuleFlag;
  int AtomicRuleFlag;
  int LogitRuleFlag;

  bool bGeneIsActive[MAX_ENV_LAYERS];
  int iGeneIndex[MAX_ENV_LAYERS];
  int iActiveGenes;

  // timming
  int iCPUTime;         // accumulated CPU time used for processing the model (in miliseconds)

  // algorithm variables
  int Resamples;

  double Accuracylimit; // The minimium post prob of a rule
  double MinUsage;
  double Mortality;

  int Totalgens;		// generations per experiment
  int Totaltrials;			// trials per experiment
  int Popsize;				// population size
  double C_rate;       // crossover rate
  double M_rate;       // mutation rate
  double J_rate;       // join rate
  double I_rate;
  double Gapsize;		// fraction of pop generated from archive
  int Trials;          // trial counter
  int Maxspin;	        // max gens without evals
  double Resampling_f;
  double Significance;   // the minimum level for inc in best
  double Conv_limit;    // the fractional addition of rules
  double Cutval;


  // data collection and loop control variables
  double  Ave_current_perf;/* ave perf in current generation       */
  double  Best;            /* best performance seen so far         */
  double  Best_current_perf;/* best perf in current generation     */
  int    Best_guy;        /* index of best_current_perf           */
  int    Conv;            /* number of partially coverged genes   */
  bool   Doneflag;        /* set when termination conditions hold */
  int    Experiment;      /* experiment counter                   */
  int    Gen;             /* generation counter                   */
  int    Lost;            /* number of totally coverged positions */
  int    Spin;            /* number of gens since eval occurred   */
  double  Worst;           /* worst performance seen so far        */
  double  Worst_current_perf;/* worst perf in current generation   */
  int     Heuristic[2][5];        /* successes of heuristic operators */
  double  Convergence;        /* the stability of rule set */
  int     Improvements;   /* the number of times the best set has been
                improved by addition or alteration */
  int Resample;

};

// ========================================================================

#endif //__GarpAlgorithm_H_
