// EnvCellSet.cpp: implementation of the EnvCellSet class.
//
// 11/19/03 Rich Williams - modified for use in SEEK
//
#include <stdio.h>
#include <stdlib.h>
#include <fstream>
#include <cstdlib>
#include <iostream>


#include "EnvCell.h"
#include "EnvCellSet.h"
#include "Utilities.h"

#include <scew.h>

// ============================================================================
EnvCellSet::EnvCellSet()
{
  intCount = 0;
  intSize = 0;
  objCells = NULL;

  histogram = NULL;

  resampled = false;
}

// ============================================================================
EnvCellSet::EnvCellSet(int size)
{
  intCount = 0;
  intSize = 0;
  objCells = NULL;

  histogram = NULL;

  resampled = false;

  initialize(size);
}

// ============================================================================
EnvCellSet::~EnvCellSet()
{
  if (histogram)
    delete histogram;

  if (objCells != NULL)
  {
    if (!resampled)
    {
      // not resampled:
      // it is not a copy of another CellSet
      // can delete all the EnvCells
      for (int i = 0; i < intCount; i++)
      {
        // EnvCellSet takes care of EnvCell property alocation and dealocation
        // possible bug!!!
        if (objCells[i])
        {
          delete[] objCells[i]->values;
          delete objCells[i];
        }
      }
    }

    delete[] objCells;
  }
}

// ============================================================================
void EnvCellSet::resampleInPlace()
{
  int p, q;
  EnvCell * oAuxCell;

  // shuffle pointers to cells
  for (int i = 0; i < intSize; i++)
  {
    // pick two random numbers
    p = GarpUtil::randint(0, intSize - 1);
    q = GarpUtil::randint(0, intSize - 1);

    // change places between cells <p> and <q>
    oAuxCell = objCells[p];
    objCells[p] = objCells[q];
    objCells[q] = oAuxCell;
  }
}

// ============================================================================
EnvCellSet * EnvCellSet::resample()
{
  // create an EnvCellSet to hold shuffled pointers to the original set
  // it is a resampled set so the cells won't be dealocate upon destruction
  // of the resampled object
  EnvCellSet * objResampled = new EnvCellSet(intSize);
  objResampled->resampled = true;

  // point to random data
  for (int i = 0; i < intSize; i++)
    objResampled->add(objCells[GarpUtil::randint(0, intSize - 1)]);

  return objResampled;
}

// ============================================================================
void EnvCellSet::initialize(int size)
{
  intCount = 0;
  intSize = size;
  objCells = new EnvCell*[intSize];
}

// ============================================================================
int EnvCellSet::genes()
{
  if (intCount)
    return objCells[0]->size();
  else
    return 0;
}

// ============================================================================
int EnvCellSet::size()
{
  return intSize;
}

// ============================================================================
int EnvCellSet::count()
{
  return intCount;
}

// ============================================================================
EnvCell * EnvCellSet::get(int index)
{
  if (index < intCount)
    return objCells[index];
  else
    throw GarpException(1, "EnvCellSet::get(): index out of bounds");

  return NULL;
}

// ============================================================================
void EnvCellSet::add(EnvCell * cell)
{
  if (intCount < intSize)
    objCells[intCount++] = cell;
  else
    throw GarpException(1, "EnvCellSet::add(): cell set already full");
}

// ============================================================================
void EnvCellSet::fromXML(char * strXML)
{
  /* sample XML
    <EnvCellSet>
      <EnvCell>0 1 2 3 4 5 6 7 8 9</EnvCell>
      <EnvCell>0 10 20 30 40 50 60 70 80 90</EnvCell>
    </EnvCellSet>
  */

  int iResult;

  scew_tree* oTree = NULL;
  scew_parser* oParser = NULL;
  oParser = scew_parser_create();

  // Loads an XML file
  iResult = scew_parser_load_buffer(oParser, (const char *) strXML, strlen(strXML));

  if (iResult)
  {
    oTree = scew_parser_tree(oParser);
    // load data from XML Node
    fromXML(scew_tree_root(oTree));
    // Remember to free tree (scew_parser_free does not free it)
    // Frees the SCEW parser
    scew_tree_free(oTree);
    scew_parser_free(oParser);
  }
  else
  {
    // Remember to free tree (scew_parser_free does not free it)
    // Frees the SCEW parser
    scew_tree_free(oTree);
    scew_parser_free(oParser);

    throw GarpException(22, "Could not load node from XML text.");
  }
}

// ============================================================================
void EnvCellSet::fromXML(scew_element const * pXMLMainNode)
{
  scew_element * pXMLSetNode, * pXMLCellNode, ** pXMLCellNodeList;

  //addref should have been called by the load method
  pXMLSetNode = scew_element_by_name_including_parent(pXMLMainNode, "EnvCellSet");

  if (pXMLSetNode)
  {
    // parse EnvCellSet node
    // get list of EnvCell nodes
    unsigned int iNumElem;
    pXMLCellNodeList = scew_element_list(pXMLSetNode, "EnvCell", &iNumElem);

    // create cells
    long i, n;
    char * strCellXML = NULL;
    n = iNumElem;
    initialize(n);

    for (i = 0; i < n; i++)
    {
      pXMLCellNode = pXMLCellNodeList[i];
      this->add(EnvCell::fromXML(pXMLCellNode));
    }

    scew_element_list_free(pXMLCellNodeList);
  }
  else
    throw GarpException(25, "XML source does not contain an EnvCellSet node.");
}

// ============================================================================
char * EnvCellSet::toXML(char * id)
{
  int i;
  int size;
  char * strXML, * strXMLCell;

  size = (4 * genes() + 128) * intCount + 2048;
  strXML = new char[size];

  if (!id)
    sprintf(strXML, "<EnvCellSet Count=\"%d\">\n", intCount);
  else
    sprintf(strXML, "<EnvCellSet Id=\"%s\" Count=\"%d\">\n", id, intCount);

  for (i = 0; i < intCount; i++)
  {
    strXMLCell = objCells[i]->toXML();
    strcat(strXML, strXMLCell);
    delete[] strXMLCell;
  }

  strcat(strXML, "</EnvCellSet>\n");

  int len = strlen(strXML);
  int diff = size - len;

  return strXML;
}

// ================================================================================

void EnvCellSet::fromXMLFile(char * strInputFilename)
{
  char *S, * strXML;
  int lFileSize;
  S = new char [4 * MAX_ENV_LAYERS + 128];
  lFileSize = GarpUtil::getFileSize(strInputFilename);
  if( lFileSize > 0 ) {
    FILE *inf = fopen(strInputFilename, "r");
    if (inf)
    {
      strXML = new char[lFileSize + MAX_ENV_LAYERS + 3];
      *strXML = 0;
      while (!feof(inf))
      {
        fscanf(inf, "%s\n", S);
        strcat(strXML, S);
        strcat(strXML, " ");
      }
      this->fromXML(strXML);
      fclose(inf);
      delete strXML;
      delete S;
    }
  }
  else
    throw GarpException(83, "Could not open XML file");
}

// ================================================================================

void EnvCellSet::toXMLFile(char * strOutputFilename, char *id)
{
  char *strXML = toXML(id);
  int iSize = strlen(strXML);
  FILE *outf = fopen(strOutputFilename, "w");
  if( outf ) {
    fwrite(strXML, sizeof(char), iSize, outf);
    fclose(outf);
    delete strXML;
  }
  else
  {
    delete strXML;
    throw GarpException(83, "Could not open XML file");
  }
}

// ==========================================================================
BioclimHistogram * EnvCellSet::getBioclimHistogram()
{ return histogram; }
// ==========================================================================
void EnvCellSet::createBioclimHistogram()
{
  //EnvCell * cell;
  int i, j, nc, ng;
  BYTE pred, val;

  if (!histogram)
    histogram = new BioclimHistogram;

  histogram->reset();

  // number of genes
  ng = genes();

  // number of cells
  nc = intCount;

  // calculate occurrence of value for each gene
  for (i = 0; i < ng; i++)
  {
    // take into account the occurrence of each value for each cell in train set
    for (j = 0; j < nc; j++)
    {
      // cell = objCells[j];

      // get presence or absence
      pred = objCells[j]->values[0];
      val  = objCells[j]->values[i];
      histogram->matrix[pred][i][val]++;
    }
  }
}

// ============================================================================
// ============================================================================
// ============================================================================
