/* An interface that find documentation.

 Copyright (c) 2006 The Regents of the University of California.
 All rights reserved.

 Permission is hereby granted, without written agreement and without
 license or royalty fees, to use, copy, modify, and distribute this
 software and its documentation for any purpose, provided that the above
 copyright notice and the following two paragraphs appear in all copies
 of this software.

 IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
 THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.

 THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
 CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 ENHANCEMENTS, OR MODIFICATIONS.
 */

package org.kepler.gui;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.LinkedList;
import java.util.List;

import ptolemy.util.ExecuteCommands;
import ptolemy.util.StringUtilities;
import ptolemy.vergil.actor.DocApplicationSpecializer;


//////////////////////////////////////////////////////////////////////////
//// KeplerDocApplicationSpecializer

/**
 Kepler specific specialization of the Ptolemy II Documentation system.

 @author Christopher Brooks
 @version $Id: KeplerDocApplicationSpecializer.java,v 1.5 2006/05/22 23:36:52 brooks Exp $
 @since Ptolemy II 6.0
 @Pt.ProposedRating Red (cxh)
 @Pt.AcceptedRating Red (cxh)
 */
public class KeplerDocApplicationSpecializer implements DocApplicationSpecializer {

    ///////////////////////////////////////////////////////////////////
    ////                         public methods                    ////

    /** Given a dot separated class name, return the URL of the PtDoc
     *  .xml file documentation or the Actor Index.  All other
     *  document types (Javadoc, source documentation) are currently
     *  not not searched for.  Thus the either the lookForPtDoc or
     *  lookForActorIndex parameter must be true, otherwise this
     *  method will return null.
     * 
     *  <p>If the KEPLER_DOCS property is set, then
     *  its value is assumed to point to the Kepler documentation.
     *  If KEPLER_DOCS is not set, then the file ./doc/docsInfo.txt
     *  is read, and if that file contains the string of the format
     *  <code>KeplerDocs: <i>path to kepler-docs</i></code> then
     *  the value of <code><i>path to kepler-docs</i></code> is used.
     *  For example, docsInfo.txt might contain
     *  <code>KeplerDocs: c:\src\kepler-docs</code>, which means
     *  that the documentation is to be found in
     *  <code>c:\src\kepler-docs</code>.  docsInfo.txt is created
     *  by running <code>cd $KEPLER; ant generateDoc</code>.
     *  The documentation for <code><i>className</i></code>
     *  is assumed to be in <code>$KEPLER_DOCS/dev/documentationFramework/generatedJavadocs/<i>className</i>.doc.xml</code>.
     *
     *  @param remoteDocumentationURLBase If non-null, the URL of the
     *  documentation.  Usually, this is set by reading the 
     *  _remoteDocumentationBase parameter from the configuration in the
     *  caller.
     *  @param className The dot separated class name.
     *  @param lookForPtDoc True if we should look for ptdoc .xml files.
     *  @param lookForJavadoc Ignored
     *  @param lookForSource Ignored
     *  @param lookForActorIndex True if we should search for the actor
     *  index *Idx.htm file.
     *  @return The URL of the documentation, if any.  If no documentation
     *  was found, return null.
     */
    public URL docClassNameToURL(String remoteDocumentationURLBase,
            String className,
            boolean lookForPtDoc, boolean lookForJavadoc,
            boolean lookForSource, boolean lookForActorIndex) {
        if (!lookForPtDoc && !lookForActorIndex) {
            return null;
        }
        URL toRead = null;
        try {
            String actorName = className.substring(className
                    .lastIndexOf('.') + 1);
            // Javadocs located under a elsewhere in Kepler
            String keplerDocsHome = System.getProperty("KEPLER_DOCS");

            if (keplerDocsHome == null) {
                // Try other way to access environment variable value.
                BufferedReader inDocsInfo = new BufferedReader(new FileReader("./doc/docsInfo.txt"));
                String info = inDocsInfo.readLine();
			
                if (info != null) {
                    String docsInfo = info.substring(info.indexOf(":") + 1);
                    docsInfo = docsInfo.replace('\\','/');
                    docsInfo = docsInfo.trim();
                    keplerDocsHome = docsInfo;
                }	
            }
            InputStream toReadStream = null;
            if (keplerDocsHome != null) {
                keplerDocsHome = keplerDocsHome.replace('\\', '/') ;

                File file = null;
                if (lookForActorIndex) {
                    file = new File(keplerDocsHome
                            + "/dev/documentationFramework/generatedJavadocs/"
                            + className.replace('.', '/') + "Idx.htm");
                } else {
                    file = new File(keplerDocsHome
                            + "/dev/documentationFramework/generatedJavadocs/"
                            + actorName + ".doc.xml");	

                }
                toRead = file.toURL();
                // Verify that we can read the URL.
                try {
                    toReadStream = toRead.openStream();
                } catch (IOException ex) {
                    toRead = null;
                } finally {
                    if (toReadStream != null) {
                        try {
                            toReadStream.close();
                        } catch (IOException ex2) {
                            // Ignore.
                        }
                    }
                }
            }
            if (toRead == null && remoteDocumentationURLBase != null) {
                try {
                    if (lookForActorIndex) {
                        toRead = new URL(remoteDocumentationURLBase
                                + className.replace('.', '/') + "Idx.htm");
                    } else {
                        toRead = new URL(remoteDocumentationURLBase
                                + actorName + ".doc.xml");
                    }
                    toReadStream = toRead.openStream();
                } catch (Exception ex) {
                    toRead = null;
                } finally {
                    if (toReadStream != null) {
                        try {
                            toReadStream.close();
                        } catch (IOException ex2) {
                            // Ignore.
                        }
                    }
                }
            }
        } catch (Throwable throwable) {
            // Ignore and return null.
            return null;
        }
        return toRead;
    }

    /** Set up the commands necessary to build the documentation.
     *  For Kepler, we run "ant generateDoc" in $KEPLER>
     *  @param executeCommands  The command execution environment necessary
     *  to build the documentation.
     *  @return A List of Strings, where each String represents the a
     *  command to be executed.
     */
    public List buildCommands(ExecuteCommands executeCommands) {
        List commands = new LinkedList();

        String osName = StringUtilities.getProperty("os.name");
        if (osName != null && osName.startsWith("Windows")) {
            commands.add("ant.bat generateDoc");
        } else {    
            commands.add("ant generateDoc");
        }
        executeCommands.setCommands(commands);

        File kepler = new File(StringUtilities.getProperty("KEPLER"));
        executeCommands.setWorkingDirectory(kepler);
        return commands;
    }
}
